"""
Testing module for VME Chat Client
"""

from .session_test_framework import SessionTestFramework, TestResult, TestCategory, PerformanceBenchmark, RegressionTests

__all__ = ['SessionTestFramework', 'TestResult', 'TestCategory', 'PerformanceBenchmark', 'RegressionTests']